# Analyze Emotions in Video

# Imports
from fer import Video
from fer import FER
import matplotlib.pyplot as plt
import os
import shutil

class Emotions:

    # Get Video File
    def initializeFiles(self, vidLocation, saveLocation):
        self.videoFile = vidLocation
        self.saveImage = saveLocation
    
    # Start Analyzing
    def analyzeVideo(self):

        # Face Detection & Video Prediction
        detector = FER(mtcnn=True)
        video = Video(self.videoFile)

        # Output List of Dictionaries
        raw_data = video.analyze(detector, display=False, save_fps=3)

        # Get Analysis Through Pandas
        df = video.to_pandas(raw_data)
        df = video.get_first_face(df)
        df = video.get_emotions(df)

        per = []
        for i in df.columns:
            per.append(df[i].sum()/(df.sum().sum())*100)
        
        # Data
        label=["Angry", "Disgust", "Fear", "Happy", "Sad", "Surprise", "Neutral"]
        plt.barh(label,per,color=["red", "pink", "orange", "cyan", "lime", "yellow", "gray"])
        plt.title("Emotion Chart")
        plt.ylabel("Emotions")
        plt.xlabel("Percentage (%)")
        plt.savefig(self.saveImage)

        print("Successfully Analyzed the Emotions!!")
     
    # Clean Up
    def cleanUp(self):
        os.remove("data.csv")
        shutil.rmtree("output")
