# Analyze Fluency in Video

# Imports
import moviepy.editor as mp
import librosa
import numpy as np
import tensorflow as tf
import plotly.graph_objects as go
import subprocess
import os

class Fluency:

    # Model H5
    new_model = tf.keras.models.load_model(str(os.getcwd()) + "/model.h5")

    # Get Video File
    def initializeFiles(self, vidLocation):
        subprocess.run("ffmpeg -r 24 -i " + vidLocation + " " + str(os.getcwd()) + "/video-final.mp4", shell=True)
        
        video = mp.VideoFileClip(str(os.getcwd()) + "/video-final.mp4")
        video.audio.write_audiofile(str(os.getcwd()) + "/audio.mp3")

        self.audioFile = str(os.getcwd()) + "/audio.mp3"
    
    # Feature Extraction
    def feature_extraction(self):
        X , sample_rate = librosa.load(str(os.getcwd()) + "/audio.mp3", sr=None)
        if X.ndim > 1:
            X = X[:,0]
            X = X.T
        
        stft = np.abs(librosa.stft(X))

        mfccs = np.mean(librosa.feature.mfcc(y=X, sr=sample_rate, n_mfcc=20).T, axis=0)
        rmse = np.mean(librosa.feature.rms(y=X).T, axis=0)
        spectral_flux = np.mean(librosa.onset.onset_strength(y=X, sr=sample_rate).T, axis=0)
        zcr = np.mean(librosa.feature.zero_crossing_rate(y=X).T, axis=0)

        return mfccs, rmse, spectral_flux, zcr
    
    # Prediction
    def prediction(self):
        mfccs, rmse, spectral_flux, zcr = Fluency().feature_extraction()

        extracted_features = np.hstack([mfccs, rmse, spectral_flux, zcr])

        pp2=extracted_features.reshape(1,-1)

        c = Fluency().new_model.predict(pp2)

        values = c
        index_max = np.argmax(values)
        return(index_max)
    
    # Visualization
    def visualization(self, l2, dd, imageFile):
        l3 = l2
        d = dd

        fig = go.Figure(go.Indicator(
            mode = "gauge",
            domain = {'x': [0, 1], 'y': [0, 1]},
            title=d,
            gauge = {
                'axis': {'range': [None, 3], 'tickwidth': 1, 'tickcolor': "darkblue"},
                'bgcolor': "white",
                'borderwidth': 2,
                'bordercolor': "gray",
                'steps':l3,
            }))

        fig.write_image(imageFile)
    
    # Start Analyzing
    def analyzeVideo(self, saveLocation):

        n = Fluency().prediction()

        if n==0:
            Fluency().visualization([{'range': [0, 1], 'color': 'red'}],{'text': "English Fluency - Beginner", 'font': {'size': 30}}, saveLocation)
        elif n==1:
            Fluency().visualization([{'range': [0, 1], 'color': 'red'},{'range': [1, 2], 'color': 'yellow'}],{'text': "English Fluency - Intermediate", 'font': {'size': 30}}, saveLocation)
        else:
            Fluency().visualization([{'range': [0, 1], 'color': 'red'},{'range': [1, 2], 'color': 'yellow'},{'range': [2, 3], 'color': 'green'}],{'text': "English Fluency - Advanced", 'font': {'size': 30}}, saveLocation)
     
    # Clean Up
    def cleanUp(self):
        os.remove(str(os.getcwd()) + "/audio.mp3")
        os.remove(str(os.getcwd()) + "/video-final.mp4")
